@@ -29,6 +29,12 @@ import urlparse
 import hmac
 import binascii
 import httplib2
+from types import ListType
+
+try:
+    from urlparse import parse_qs, parse_qsl
+except ImportError:
+    from cgi import parse_qs, parse_qsl
 
 
 VERSION = '1.0' # Hi Blaine!
@@ -192,7 +198,7 @@ class Token(object):
         if not len(s):
             raise ValueError("Invalid parameter string.")
 
-        params = urlparse.parse_qs(s, keep_blank_values=False)
+        params = parse_qs(s, keep_blank_values=False)
         if not len(params):
             raise ValueError("Invalid parameter string.")
 
@@ -304,7 +310,10 @@ class Request(dict):
  
     def to_postdata(self):
         """Serialize as post data for a POST request."""
-        return urllib.urlencode(self)
+        # tell urlencode to deal with sequence values and map them correctly
+        # to resulting querystring. for example self["k"] = ["v1", "v2"] will
+        # result in 'k=v1&k=v2' and not k=%5B%27v1%27%2C+%27v2%27%5D
+        return urllib.urlencode(self, True)
  
     def to_url(self):
         """Serialize as a URL for a GET request."""
@@ -319,8 +328,14 @@ class Request(dict):
  
     def get_normalized_parameters(self):
         """Return a string that contains the parameters that must be signed."""
-        items = [(k, v) for k, v in self.items() if k != 'oauth_signature']
-        return urllib.urlencode(sorted(items))
+        # 1.0a/9.1.1 states that kvp must be sorted by key, then by value
+        items = [(k, v if type(v) != ListType else sorted(v)) for k,v in sorted(self.items()) if k != 'oauth_signature']
+        encoded_str = urllib.urlencode(items, True)
+        # Encode signature parameters per Oauth Core 1.0 protocol
+        # spec draft 7, section 3.6
+        # (http://tools.ietf.org/html/draft-hammer-oauth-07#section-3.6)
+        # Spaces must be encoded with "%20" instead of "+"
+        return encoded_str.replace('+', '%20')
  
     def sign_request(self, signature_method, consumer, token):
         """Set the signature parameter to the result of sign."""
@@ -435,7 +450,7 @@ class Request(dict):
     @staticmethod
     def _split_url_string(param_str):
         """Turn URL string into parameters."""
-        parameters = urlparse.parse_qs(param_str, keep_blank_values=False)
+        parameters = parse_qs(param_str, keep_blank_values=False)
         for k, v in parameters.iteritems():
             parameters[k] = urllib.unquote(v[0])
         return parameters
@@ -554,7 +569,7 @@ class Client(httplib2.Http):
         httplib2.Http.__init__(self, cache=cache, timeout=timeout, 
             proxy_info=proxy_info)
 
-    def set_signature_method(method):
+    def set_signature_method(self, method):
         if not isinstance(method, SignatureMethod):
             raise ValueError("Invalid signature method.")
 
@@ -567,10 +582,10 @@ class Client(httplib2.Http):
             headers = {}
 
         if body and method == "POST":
-            parameters = dict(urlparse.parse_qsl(body))
+            parameters = dict(parse_qsl(body))
         elif method == "GET":
             parsed = urlparse.urlparse(uri)
-            parameters = urlparse.parse_qs(parsed.query)     
+            parameters = parse_qsl(parsed.query)
         else:
             parameters = None
 
